"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cli = void 0;
const chalk_1 = require("chalk");
// @ts-ignore when building
const package_json_1 = __importDefault(require("../package.json"));
const replaceBits_1 = require("./replaceBits");
async function cli(args = process.argv.slice(2)) {
    const { help, version } = {
        ...consumeOptionArgument('help', { short: 'h' }),
        ...consumeOptionArgument('version', { short: 'v' }),
    };
    function consumeOptionArgument(name, { short }) {
        var _a;
        let i = args.indexOf(`--${name}`);
        if (i === -1 && short) {
            i = args.indexOf(`-${short}`);
        }
        return i !== -1 ? { [name]: (_a = args.splice(i, 2)[1]) !== null && _a !== void 0 ? _a : true } : {};
    }
    if (help) {
        showHelp();
        return;
    }
    if (version) {
        console.log(`${package_json_1.default.name}@${package_json_1.default.version}`);
        return;
    }
    function showHelp() {
        console.log(package_json_1.default.description);
        console.log();
        console.log('Usage:');
        console.log(`  ${package_json_1.default.name} [options]`);
        console.log();
        console.log('Options:');
        console.log(`  -h, --help`);
        console.log(`  -v, --version`);
        /* prettier-ignore */
        console.log(`  -b, --bitsDirName  default: "${replaceBits_1.DefaultOptions.BitsDirName}"`);
        /* prettier-ignore */
        console.log(`  -r, --root         default: "${replaceBits_1.DefaultOptions.Root}"`);
        /* prettier-ignore */
        console.log(`  -o, --outputDir    default: "${replaceBits_1.DefaultOptions.OutputDir}"`);
    }
    const replaceOptions = {
        ...consumeOptionArgument('bitsDirName', { short: 'b' }),
        ...consumeOptionArgument('root', { short: 'r' }),
    };
    const writeOptions = {
        ...consumeOptionArgument('outputDir', { short: 'o' }),
    };
    if (args.length) {
        bail('too many arguments');
        return;
    }
    function bail(message) {
        console.error(chalk_1.red(message));
        console.log();
        showHelp();
    }
    try {
        await replaceBits_1.writeResult(replaceBits_1.replaceBits(replaceOptions), writeOptions);
    }
    catch (error) {
        console.error(error.message);
        console.log();
    }
}
exports.cli = cli;
//# sourceMappingURL=cli.js.map