"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeResult = exports.replaceBits = exports.DefaultOptions = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const util_1 = require("util");
const chalk_1 = require("chalk");
const globby_1 = __importDefault(require("globby"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const readFile = util_1.promisify(fs_1.readFile);
const writeFile = util_1.promisify(fs_1.writeFile);
var DefaultOptions;
(function (DefaultOptions) {
    DefaultOptions["BitsDirName"] = "_bits";
    DefaultOptions["Root"] = "./documents";
    DefaultOptions["OutputDir"] = "./dist/documents";
})(DefaultOptions = exports.DefaultOptions || (exports.DefaultOptions = {}));
/* istanbul ignore next */
/**
 * Replaces markdown bits in Markdown files that live within `${options.root}`.
 * Searches for bits that match the format `${bit-name}` with content
 * from `${options.bitsDirName}/${bit-name}.md` within the same folder.
 * @param options
 */
async function* replaceBits({ bitsDirName = DefaultOptions.BitsDirName, root = DefaultOptions.Root, } = {}) {
    const bitPathsGlob = path_1.posix.join(root, `**/${bitsDirName}/*`);
    const bits = await loadBits();
    async function loadBits() {
        const bits = new Map();
        const bitPaths = await globby_1.default(bitPathsGlob);
        await Promise.all(bitPaths.map(async (bitPath) => {
            const bp = bitPath.toString();
            bits.set(path_1.posix.join(path_1.posix.dirname(bp), path_1.posix.basename(bp)), (await readFile(bp)).toString());
        }));
        return bits;
    }
    for await (const docPath of globby_1.default.stream([
        path_1.posix.join(root, '**'),
        `!${bitPathsGlob}`,
    ])) {
        const bitPattern = /\${([\w\d-]+)}/g;
        const docDir = path_1.posix.dirname(docPath.toString());
        const contents = (await readFile(docPath)).toString();
        yield [
            path_1.posix.relative(root, docPath.toString()),
            contents.replace(bitPattern, (_m, bitName) => {
                const absoluteRoot = path_1.posix.resolve(root);
                let currentDir = docDir;
                while (true) {
                    const bitPath = path_1.posix.join(currentDir, bitsDirName, `${bitName}.md`);
                    const bit = bits.get(bitPath);
                    if (bit) {
                        return bit;
                    }
                    const parentDir = path_1.posix.resolve(currentDir, '..');
                    if (currentDir === absoluteRoot ||
                        currentDir === parentDir) {
                        bail();
                    }
                    currentDir = parentDir;
                }
                function bail() {
                    throw new Error(chalk_1.red(`bit \${${bitName}} not found relative to ${path_1.posix.relative(process.cwd(), docDir)}`));
                }
            }),
        ];
    }
}
exports.replaceBits = replaceBits;
async function writeResult(files, 
/* istanbul ignore next */
{ outputDir = DefaultOptions.OutputDir } = {}) {
    await mkdirp_1.default(outputDir);
    const writes = [];
    for await (const [relativePath, contents] of files) {
        const outputPath = path_1.resolve(outputDir, relativePath);
        writes.push(mkdirp_1.default(path_1.dirname(outputPath)).then(() => writeFile(outputPath, contents)));
    }
    return Promise.all(writes);
}
exports.writeResult = writeResult;
//# sourceMappingURL=replaceBits.js.map